<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Admin;

use Slim\App;
use MicroEngine\Admin\Middleware\AuthMiddleware;
use MicroEngine\Core\Middleware\DBTransactionMiddleware;
use MicroEngine\Install\Middleware\SchemaVersionMiddleware;

class AdminRoute
{
    /**
     * @param  App $app
     */
    public function register(App $app)
    {
        $c = $app->getContainer();

        $app->group('/admin', function () {
            // ホーム
            $this->get('', 'homeController:index')->setName('admin.home.index');
            $this->get('/portlet', 'homeController:portlet')->setName('admin.home.portlet');
            $this->put('/portlet', 'homeController:updatePortlet')->setName('admin.home.updatePortlet');
            // ユーザー
            $this->group('/user', function () {
                $this->get('[/page/{page_no:[0-9]+}]', 'userController:index')->setName('admin.user.index');
                $this->get('/create', 'userController:create')->setName('admin.user.create');
                $this->post('', 'userController:store')->setName('admin.user.store');
                $this->get('/{id}/edit', 'userController:edit')->setName('admin.user.edit');
                $this->put('/{id}', 'userController:update')->setName('admin.user.update');
                $this->put('/unlock/{id}', 'userController:unlock')->setName('admin.user.unlock');
                $this->delete('/{id}', 'userController:destroy')->setName('admin.user.destroy');
            });
            // ブックマーク
            $this->group('/bookmark', function () {
                $this->get('', 'bookmarkController:index')->setName('admin.bookmark.index');
                $this->get('/edit', 'bookmarkController:edit')->setName('admin.bookmark.edit');
                $this->put('', 'bookmarkController:update')->setName('admin.bookmark.update');
            });
            // 設定
            $this->group('/config', function () {
                $this->get('', 'configController:index')->setName('admin.config.index');
                $this->get('/{filename}/edit', 'configController:edit')->setName('admin.config.edit');
                $this->put('/{filename}', 'configController:update')->setName('admin.config.update');
            });
            // ライセンス
            $this->group('/license', function () {
                $this->get('', 'licenseController:index')->setName('admin.license.index');
                $this->get('/register', 'licenseController:register')->setName('admin.license.register');
                $this->post('', 'licenseController:store')->setName('admin.license.store');
            });
            // ログ
            $this->get('/log/viewer[/{filename}]', 'logController:viewer')->setName('admin.log.viewer');
            // システム情報
            $this->group('/system', function () {
                $this->get('', 'systemController:index')->setName('admin.system.index');
                $this->get('/phpinfo', 'systemController:phpinfo')->setName('admin.system.phpinfo');
                $this->get('/libs', 'systemController:libs')->setName('admin.system.libs');
            });
        })->add(new AuthMiddleware($c))->add(new SchemaVersionMiddleware($c))->add(new DBTransactionMiddleware($c));

        // ログイン・ログアウト
        $app->group('/admin/auth', function () {
            $this->get('/login', 'authController:login')->setName('admin.auth.login');
            $this->post('/login', 'authController:attemptLogin');
            $this->post('/logout', 'authController:logout')->setName('admin.auth.logout');
        })->add(new SchemaVersionMiddleware($c))->add(new DBTransactionMiddleware($c));
    }
}
