<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Admin\Controllers;

use Psr\Http\Message\ResponseInterface;
use MicroEngine\Admin\Forms\PortletForm;
use Psr\Http\Message\ServerRequestInterface;
use MicroEngine\Core\Controllers\Controller;

class HomeController extends Controller
{
    /**
     * @param  ServerRequestInterface $request
     * @param  ResponseInterface $response
     * @param  array $args
     * @return ResponseInterface
     */
    public function index(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $user = $this->c->auth->getUser();
        $portlets = $user->portlets;

        return $this->view->render(
            $response,
            'admin/views/home/index.twig',
            compact('portlets')
        );
    }

    /**
     * @param  ServerRequestInterface $request
     * @param  ResponseInterface $response
     * @param  array $args
     * @return ResponseInterface
     */
    public function portlet(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $user = $this->c->auth->getUser();
        $portlets = $user->portlets;
        $portletsConfig = config('admin.portlets');

        return $this->view->render(
            $response,
            'admin/views/home/portlet.twig',
            compact('portlets', 'portletsConfig')
        );
    }

    /**
     * @param  ServerRequestInterface $request
     * @param  ResponseInterface $response
     * @param  array $args
     * @return ResponseInterface
     * @throws \Rakit\Validation\RuleQuashException
     */
    public function updatePortlet(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $form = new PortletForm();
        $form->setInputValue($request->getParsedBody());

        // バリデーション
        if ($form->validate()) {
            $id = $this->c->auth->getUserId();
            $this->c->userManager->updatePortlets($id, $form->getInputValues());

            return $this->view->render(
                $response,
                'admin/views/partials/success.twig',
                ['message' => 'ポートレット設定を更新しました。']
            );
        } else {
            return $this->view->render(
                $response->withStatus(400),
                'admin/views/partials/danger.twig',
                ['message' => 'ポートレット設定の更新に失敗しました。']
            );
        }
    }
}
