<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Admin\Services;

use MicroEngine\Core\Support\FileSystem;
use Interop\Container\ContainerInterface;

class Log
{
    /**
     * @var ContainerInterface
     */
    protected $c;
    /**
     * @var FileSystem
     */
    protected $fileSystem;

    /**
     * @param ContainerInterface $c
     */
    public function __construct(ContainerInterface $c)
    {
        $this->c = $c;
        $this->fileSystem = new FileSystem(system_root('storage/logs'));
    }

    /**
     * ログファイル一覧を取得
     *
     * @return array
     * @throws \MicroEngine\Core\Exception\MeException
     */
    public function files(): array
    {
        $files = $this->fileSystem->globFiles('*.log');

        return $files;
    }

    /**
     * ログファイルの内容を取得
     *
     * @param string $filename
     * @return string $content
     * @throws \MicroEngine\Core\Exception\MeException
     */
    public function content(string $filename): string
    {
        $content = '';
        if ($this->fileSystem->has($filename)) {
            $content = $this->fileSystem->get($filename);
        }
        return $content;
    }

    /**
     * ファイルの末尾を取得
     *
     * @param string $filename
     * @param int $numberLines
     * @return string
     * @throws \MicroEngine\Core\Exception\MeException
     */
    public function tail(string $filename, int $numberLines = 10): string
    {
        return $this->fileSystem->tail($filename, $numberLines);
    }
}
