<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Cms\Calendar;

use MicroEngine\Core\Support\TermLink;
use Illuminate\Database\Eloquent\Builder;
use MicroEngine\Cms\Archive\MonthlyArchive;

trait CalendarTrait
{
    /**
     * 月カレンダースコープ
     *
     * @param Builder $query
     * @param array $options
     * @param string|null $year
     * @param string|null $month
     * @return MonthlyArchive
     */
    public function scopeMonthlyCalendar(Builder $query, array $options, string $year = null, string $month = null): MonthlyArchive
    {
        // 設定の初期値をセット
        $options += [
            'month' => 1,
            'offset_month' => 0,
            'sort' => 'asc',
            'blank' => 1,
            'link_format' => 'Y年m月',
        ];

        // 期間リストを取得
        $termList = $query->termList()->get();

        // リンクリストを取得
        $termLink = new TermLink();
        $termLink->setTerm($termList, $options);

        // indexページの場合は $year, $month はわたされないので、リンク情報取得時に判定した年月をセットする
        if ($year === null) {
            $year = $termLink->getYear();
        }
        if ($month === null) {
            $month = $termLink->getMonth();
        }

        // アーカイブオブジェクトを作成する
        return new MonthlyArchive(collect(), $termLink->getLinks(), $options, $year, $month);
    }
}
