<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Cms;

use MicroEngine\Cms\Views\CmsExtension;
use MicroEngine\Core\Support\ServiceProvider;
use MicroEngine\Cms\Services\{
    Backup, Data, CodeEditor, Import, ImageUtility, Theme, ContentService
};
use MicroEngine\Cms\Managers\{
    FileManager, PageManager, TableManager, CategoryManager, CalendarManager, CollectionManager, ModuleManager
};
use MicroEngine\Cms\Controllers\{
    BackupController, CacheController, PageController, FileController, FieldController, TableController,
    ThemeController, ModuleController, ContentController, CalendarController, CategoryController, CollectionController
};

/**
 * @package MicroEngine\Cms
 */
class CmsServiceProvider extends ServiceProvider
{
    /**
     * サービス登録
     */
    public function register()
    {
        $c = $this->c;

        $c['categoryManager'] = function ($c) {
            return new CategoryManager($c);
        };
        $c['pageManager'] = function ($c) {
            return new PageManager($c);
        };
        $c['collectionManager'] = function ($c) {
            return new CollectionManager($c);
        };
        $c['moduleManager'] = function ($c) {
            return new ModuleManager($c);
        };
        $c['calendarManager'] = function ($c) {
            return new CalendarManager($c);
        };
        $c['contentService'] = function ($c) {
            return new ContentService($c);
        };
        $c['fileManager'] = function ($c) {
            return new FileManager($c);
        };
        $c['tableManager'] = function ($c) {
            return new TableManager($c);
        };
        $c['theme'] = function ($c) {
            return new Theme($c);
        };
        $c['backup'] = function ($c) {
            return new Backup($c);
        };
        $c['import'] = function ($c) {
            return new Import($c);
        };
        $c['data'] = function ($c) {
            return new Data($c);
        };
        $c['imageUtility'] = function ($c) {
            return new ImageUtility($c);
        };
        $c['codeEditor'] = function ($c) {
            return new CodeEditor($c);
        };
        $c['categoryController'] = function ($c) {
            return new CategoryController($c);
        };
        $c['pageController'] = function ($c) {
            return new PageController($c);
        };
        $c['collectionController'] = function ($c) {
            return new CollectionController($c);
        };
        $c['moduleController'] = function ($c) {
            return new ModuleController($c);
        };
        $c['calendarController'] = function ($c) {
            return new CalendarController($c);
        };
        $c['fieldController'] = function ($c) {
            return new FieldController($c);
        };
        $c['tableController'] = function ($c) {
            return new TableController($c);
        };
        $c['fileController'] = function ($c) {
            return new FileController($c);
        };
        $c['themeController'] = function ($c) {
            return new ThemeController($c);
        };
        $c['backupController'] = function ($c) {
            return new BackupController($c);
        };
        $c['cacheController'] = function ($c) {
            return new CacheController($c);
        };
    }

    /**
     * サービス起動
     */
    public function boot()
    {
        $c = $this->c;

        // Twigエクステンションを登録
        $c->view->addExtension(new CmsExtension($c));

        // Register routes
        $route = new CmsRoute();
        $route->register($this->app);
    }
}
