<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Core\Handler;

use Interop\Container\ContainerInterface;
use Slim\Handlers\NotFound as SlimNotFound;
use Psr\Http\Message\ServerRequestInterface;

class NotFound extends SlimNotFound
{
    use HandlerTrait;

    /**
     * @var ContainerInterface
     */
    protected $c;

    /**
     * @param ContainerInterface $c
     */
    public function __construct(ContainerInterface $c)
    {
        $this->c = $c;
    }

    /**
     * text/html コンテンツの not found レスポンスを返す
     *
     * @param  ServerRequestInterface $request The most recent Request object
     * @return ResponseInterface
     */
    protected function renderHtmlNotFoundOutput(ServerRequestInterface $request)
    {
        $title = 'ページが見つかりません / 404 Page Not Found';
        $homeUrl = (string)($request->getUri()->withPath('')->withQuery('')->withFragment(''));

        $view = $this->c->view;
        // 公開テーマに 404テンプレートがないか確認する
        $templatePath = config('settings.template_404', '@site/page/404.twig');
        if ($view->getLoader()->exists($templatePath)) {
            $page = [
                'name' => $title,
            ];
            $html = $this->c->view->fetch($templatePath, compact('page', 'homeUrl'));

            return $html;
        }

        $html = '<p>お探しのページを見つけることができませんでした。ブラウザのアドレスバーのURLが正しいことを確認してください。</p>'
            . '<main><a href="' . e($homeUrl) . '">トップページに移動する</a></main>';

        return $this->renderTemplate($title, $html);
    }
}
