<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Core\Middleware;

use Psr\Http\Message\ResponseInterface;
use MicroEngine\Core\Exception\MeException;
use Psr\Http\Message\ServerRequestInterface;

/**
 * 外部入力値のバリデーション
 * クエリパラメータ・POSTデータ・パスパラメータの文字エンコーディングが正しいことを確認する
 */
class InputValidationMiddleware extends Middleware
{
    /**
     * @param  ServerRequestInterface $request
     * @param  ResponseInterface $response
     * @param  callable $next
     * @return ResponseInterface
     * @throws \Exception
     */
    public function __invoke(ServerRequestInterface $request, ResponseInterface $response, callable $next): ResponseInterface
    {
        // クエリパラメータを確認
        $this->checkEncoding($request->getQueryParams());
        // POSTデータを確認
        $data = $request->getParsedBody();
        if ($data !== null) {
            $this->checkEncoding($data);
        }
        // パスパラメータを確認
        $route = $request->getAttribute('route');
        $args = $route->getArguments();
        $this->checkEncoding($args);

        return $next($request, $response);
    }

    /**
     * 文字エンコーディングが正しいことを検証する
     *
     * @param array $inputs
     * @throws MeException
     */
    protected function checkEncoding(array $inputs)
    {
        foreach ($inputs as $input) {
            if (!mb_check_encoding($input, 'UTF-8')) {
                throw new MeException('入力値の文字コード検証に失敗しました。');
            }
        }
    }
}
