<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Core\Pagination;

use Illuminate\Database\Eloquent\Builder;

trait PaginationTrait
{
    /**
     * ページネーションスコープ
     *
     * @param Builder $query
     * @param int $currentPage
     * @param array $options
     * @return Pagination
     */
    public function scopePagination(Builder $query, int $currentPage = 1, array $options = []): Pagination
    {
        // 設定値を初期値としてセットする
        $options += config('settings.pagination');
        $total = $query->count();
        $offset = ($currentPage - 1) * $options['per_page'];
        $items = $query->offset($offset)->limit($options['per_page'])->get();
        $pagination = new Pagination($items, $total, $currentPage, $options);

        return $pagination;
    }
}
