<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Core\Validation;

use Rakit\Validation\Rule;

class DirPath extends Rule
{
    /**
     * @var string
     */
    protected $message = ':attributeは半角英数字と一部の記号（-_.）のみの文字列を / で囲って指定してください。';

    /**
     * @param mixed $value
     * @return bool
     */
    public function check($value)
    {
        // スラッシュ一文字の場合はOK
        if ($value === '/') {
            return true;
        }

        // １文字目と最後の１文字が / であることを確認する
        if (substr($value, 0, 1) !== '/' || substr($value, -1) !== '/') {
            // 一つでも一致しなければfalseを返す
            return false;
        }

        // 前後のスラッシュを取り除き、/で分解する
        $dirs = explode('/', trim($value, '/'));

        foreach ($dirs as $dir) {
            if (preg_match('/\A[a-zA-Z0-9-_.]+\z/u', $dir) !== 1) {
                // 一つでも一致しなければfalseを返す
                return false;
            }
        }
        return true;
    }
}
