<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Install\Controllers;

use Psr\Http\Message\ResponseInterface;
use MicroEngine\Core\Exception\MeException;
use Psr\Http\Message\ServerRequestInterface;
use MicroEngine\Core\Controllers\Controller;
use MicroEngine\Install\Services\UpgradeService;

class UpgradeController extends Controller
{
    /**
     * @param  ServerRequestInterface $request
     * @param  ResponseInterface $response
     * @param  array $args
     * @return ResponseInterface
     * @throws \Exception
     */
    public function index(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        /** @var  $upgradeService UpgradeService */
        $upgradeService = $this->c->upgradeService;

        // アップグレードの実行可否確認
        if (!$upgradeService->needsUpgrade()) {
            throw new MeException('アップグレードは実行できません。');
        }

        // DBのスキーマバージョン値を取得
        $dbVersion = $upgradeService->dbVersion();
        // ソースコード側のスキーマバージョン値を取得
        $codeVersion = config('settings.schema_version');

        return $this->view->render(
            $response,
            'install/views/upgrade/index.twig',
            compact('dbVersion', 'codeVersion')
        );
    }

    /**
     * @param  ServerRequestInterface $request
     * @param  ResponseInterface $response
     * @param  array $args
     * @return ResponseInterface
     * @throws \Exception
     */
    public function upgrade(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        /** @var  $upgradeService UpgradeService */
        $upgradeService = $this->c->upgradeService;

        // アップグレードの実行可否確認
        if (!$upgradeService->needsUpgrade()) {
            throw new MeException('アップグレードは実行できません。');
        }

        // DBのスキーマバージョン値を取得
        $dbVersion = $upgradeService->dbVersion();
        // ソースコード側のスキーマバージョン値を取得
        $codeVersion = config('settings.schema_version');

        // アップグレード実行
        $upgradeService->upgrade($dbVersion, $codeVersion);

        return $this->view->render(
            $response,
            'install/views/upgrade/upgrade.twig',
            compact('dbVersion', 'codeVersion')
        );
    }
}
