<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Install;

use Slim\App;
use MicroEngine\Install\Middleware\InstalledMiddleware;
use MicroEngine\Core\Middleware\DBTransactionMiddleware;

class InstallRoute
{
    /**
     * @param App $app
     */
    public function register(App $app)
    {
        $c = $app->getContainer();

        // インストール
        $app->group('/admin/install', function () use ($c) {
            $this->get('', 'installController:start')->setName('admin.install.start');
            $this->get('/eula', 'installController:eula')->setName('admin.install.eula');
            $this->get('/env', 'installController:env')->setName('admin.install.env');
            $this->get('/database', 'installController:createDatabase')->setName('admin.install.createDatabase');
            $this->post('/database', 'installController:storeDatabase')->setName('admin.install.storeDatabase');
            $this->get('/user', 'installController:createUser')->setName('admin.install.createUser');
            $this->post('/user', 'installController:storeUser')->setName('admin.install.storeUser')
                ->add(new DBTransactionMiddleware($c));
        })->add(new InstalledMiddleware($c));

        // アップグレード
        $app->group('/admin/upgrade', function () use ($c) {
            $this->get('', 'upgradeController:index')->setName('admin.upgrade.index');
            $this->post('/upgrade', 'upgradeController:upgrade')->setName('admin.upgrade.upgrade');
        })->add(new DBTransactionMiddleware($c));
    }
}
