<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Site\Controllers;

use MicroEngine\Cms\Models\Page;
use Slim\Exception\NotFoundException;
use MicroEngine\Cms\Models\Collection;
use Psr\Http\Message\ResponseInterface;
use MicroEngine\Core\Controllers\Action;
use Psr\Http\Message\ServerRequestInterface;

class ArchiveAction extends Action
{
    /**
     * @param  ServerRequestInterface $request
     * @param  ResponseInterface $response
     * @param  array $args
     * @param  Page|null $page
     * @return ResponseInterface
     * @throws NotFoundException
     */
    public function __invoke(ServerRequestInterface $request, ResponseInterface $response, array $args, Page $page = null): ResponseInterface
    {
        $year = $args['year'] ?? null;
        $month = $args['month'] ?? null;

        // ルート名・タイプを取得
        $routeName = $request->getAttribute('route')->getName();
        if ($routeName !== null) {
            $type = $routeName;
        } else {
            // RoutingActionから転送された場合
            $type = $page->type;
            $routeName = $type;
        }
        // 設定を取得
        $config = config('pages.' . $type);

        // 該当コンテンツのオプション設定を取得
        $options = $config['options'];
        $baseType = $options['base_type'] ?? 'page';
        $targetContent = $options['content'];

        if ($baseType === 'page') {
            $archive = Page::where('type', $targetContent)->monthlyArchive($options, $year, $month);
        } elseif ($baseType === 'collection') {
            $archive = Collection::where('type', $targetContent)->monthlyArchive($options, $year, $month);
        } else {
            throw new NotFoundException($request, $response);
        }
        // ルート情報をセット
        $archive->setRouteInfo($this->c, $routeName);

        if ($archive->outOfBounds()) {
            throw new NotFoundException($request, $response);
        }

        // 何ページ目を表示する場合でも、１ページ目の情報をDBから取得する
        if (!isset($page)) {
            $page = Page::where('type', $type)->first();
            if (!$page) {
                throw new NotFoundException($request, $response);
            }
        }
        // １ページ目のパスをセット
        $archive->setFirstPath($page->path);
        $this->c->site->pushContent($page, true);

        return $this->c->view->render(
            $response,
            $page->getTemplatePath(),
            compact('archive', 'page')
        );
    }
}
