<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Site\Controllers;

use MicroEngine\Cms\Models\Page;
use MicroEngine\Cms\Models\Calendar;
use Slim\Exception\NotFoundException;
use Psr\Http\Message\ResponseInterface;
use MicroEngine\Core\Controllers\Action;
use MicroEngine\Cms\Archive\MonthlyArchive;
use Psr\Http\Message\ServerRequestInterface;

class CalendarAction extends Action
{
    /**
     * @param  ServerRequestInterface $request
     * @param  ResponseInterface $response
     * @param  array $args
     * @param  Page|null $page
     * @return ResponseInterface
     * @throws NotFoundException
     */
    public function __invoke(ServerRequestInterface $request, ResponseInterface $response, array $args, Page $page = null): ResponseInterface
    {
        $year = $args['year'] ?? null;
        $month = $args['month'] ?? null;

        // ルート名・タイプを取得
        $routeName = $request->getAttribute('route')->getName();
        if ($routeName !== null) {
            $type = $routeName;
        } else {
            // RoutingActionから転送された場合
            $type = $page->type;
            $routeName = $type;
        }
        // 設定を取得
        $config = config('pages.' . $type);

        // 該当コンテンツのオプション設定を取得
        $options = $config['options'];
        $targetContent = $options['content'];
        $targetConfig = config('calendars.' . $targetContent);

        if (!empty($targetConfig)) {
            /* @var MonthlyArchive $archive */
            $archive = Calendar::where('type', $targetContent)->monthlyCalendar($options, $year, $month);
        } else {
            throw new NotFoundException($request, $response);
        }
        $archive->setRouteInfo($this->c, $routeName);

        // パラメータがある場合は範囲外かどうか確認する
        if ($year !== null && $month !== null && $archive->outOfBounds()) {
            throw new NotFoundException($request, $response);
        }

        // 何ページ目を表示する場合でも、１ページ目の情報をDBから取得する
        if (!isset($page)) {
            $page = Page::where('type', $type)->first();
            if (!$page) {
                throw new NotFoundException($request, $response);
            }
        }
        // １ページ目のパスをセット
        $archive->setFirstPath($page->path);
        $this->c->site->pushContent($page, true);
        $options = $archive->getOptions();

        return $this->c->view->render(
            $response,
            $page->getTemplatePath(),
            compact('archive', 'page', 'options')
        );
    }
}
