<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

global $app;
$app = new \Slim\App();
$c = $app->getContainer();

// settings は最初にセットする
$settings = $c->settings;
// app_settings をベースに settings で上書きする。
$settings->replace(array_merge(require(__DIR__ . '/app_settings.php'), require(__DIR__ . '/../config/settings.php')));

// 定数定義
define('SOFTWARE_VERSION', $settings->get('version'));
define('SOFTWARE_COPYRIGHT_YEAR', $settings->get('copyright_year'));

// 詳細エラー表示設定。このファイルの読み込み元の index.php で使われるために定義する。
$displayErrorDetails = $settings->get('displayErrorDetails', false);

// 詳細エラー表示をしない場合は、PHP の display_errors に 0 をセットしてエラーをHTMLに出力しないようにする。
if (!$displayErrorDetails) {
    ini_set('display_errors', 0);
}

// PHP設定
foreach ($settings->get('ini_set', []) as $iniName => $iniValue) {
    ini_set($iniName, $iniValue);
}

// タイムゾーンの設定
date_default_timezone_set($settings->get('timezone', 'Asia/Tokyo'));

// ロケール設定
setlocale(LC_ALL, $settings->get('locale', 'ja_JP.UTF-8'));

// デフォルト日付フォーマット設定
\Carbon\Carbon::setToStringFormat($settings->get('date_default', 'Y/m/d H:i:s'));

// サービスプロバイダを登録する
$serviceNames = $settings->get('services', []);
$services = [];

// 各サービスの register 処理
foreach ($serviceNames as $serviceName) {
    $serviceProviderClass = '\\MicroEngine\\' . $serviceName . '\\' . $serviceName . 'ServiceProvider';
    $serviceProvider = new $serviceProviderClass($app);
    $serviceProvider->register();
    $services[$serviceName] = $serviceProvider;
}

// 各パッケージの boot 処理
foreach ($services as $serviceProvider) {
    $serviceProvider->boot();
}

// Register routes
if (file_exists(__DIR__ . '/../app/routes.php')) {
    require __DIR__ . '/../app/routes.php';
}

// アプリケーションを返す
return $app;
