/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

(function ($, window) {
  'use strict';

  if (typeof window.mecms === 'undefined') {
    window.mecms = {};
  }

  /**
   * Table
   */
  mecms.table = {
    $block: null,
    init: function () {
      var self = this, rows;

      $('#addRow').on('click', function (e) {
        self.addRow(e);
      });
      $('#addField').on('click', function (e) {
        self.addRow(e);
      });
      $('#dataRows').on('focus', 'tr', function (e) {
        self.selectBlock(e);
      });
      $('.content').on('click', function (e) {
        self.deselectBlock(e);
      });
      $('#containerToolbox').on('keydown', function (e) {
        self.keyDown(e);
      });
      $('#upField').on('click', function (e) {
        self.up(e);
      });
      $('#downField').on('click', function (e) {
        self.down(e);
      });
      $('#removeField').on('click', function (e) {
        self.remove(e);
      });
      $('#dataRows').on('keypress', 'input', function (e) {
        self.addRowByEnter(e);
      });
      $('#tsvEditor').on('click', function (e) {
        self.openCodeEditor(e);
      });
      $('#codeModal').on('hidden.bs.modal', function (e) {
        self.closeCodeEditor(e);
      });
      $('#tableForm').on('submit', function (e) {
        self.submitForm(e);
      });
      rows = JSON.parse($('#data').val());
      self.updateRows(rows);
    },
    submitForm: function (e) {
      var self = this, rows;

      rows = self.currentRows();
      $('#data').val(JSON.stringify(rows));
    },
    currentRows: function () {
      var rows = [], cols, $input;

      $('#dataRows > tr').each(function () {
        cols = [];
        $(this).find('input').each(function () {
          $input = $(this);
          cols.push($input.val());
        });
        rows.push(cols);
      });
      return rows;
    },
    openCodeEditor: function (e) {
      var self = this, $codeEditor, rows, lines = [], tsv;

      $('#codeModal').modal('show');
      rows = self.currentRows();
      $.each(rows, function (index, cols) {
        lines.push(cols.join("\t"));
      });
      tsv = lines.join("\n");

      mecms.codeEditor.startCodeMirror([
        'codemirror',
        'codemirror/addon/dialog/dialog',
        'codemirror/addon/search/search',
        'codemirror/addon/search/searchcursor',
        'codemirror/addon/search/jump-to-line'
      ], 'null', tsv);
    },
    closeCodeEditor: function (e) {
      var self = this, text, rows = [], cols, lines;

      text = mecms.codeEditor.codeMirror.getDoc().getValue();
      lines = text.split("\n");
      $.each(lines, function (index, line) {
        cols = line.split("\t");
        rows.push(cols);
      });
      self.updateRows(rows);
    },
    updateRows: function (rows) {
      var self = this, $newRows, $cloned, i, $inputs, $dataRows;

      $dataRows = $('#dataRows').empty();
      $newRows = $('#newRows');

      $.each(rows, function (key, row) {
        $cloned = $newRows.children().clone();
        $dataRows.append($cloned);
        $inputs = $cloned.find('input');
        $.each(row, function (index, val) {
          $inputs.eq(index).val(val);
        });
      })
    },
    addRowByEnter: function (e) {
      var self = this;

      if (e.keyCode !== 13) {
        return true;
      }
      return self.addRow(e);
    },
    addRow: function (e) {
      var self = this, $target, $newRows, $dataRows, $cloned;

      $target = $(e.currentTarget);
      $newRows = $('#newRows');
      $dataRows = $('#dataRows');
      $cloned = $newRows.children().clone();
      if ($target.is('input')) {
        $target.parents('tr').after($cloned);
      } else if ($target.is('#addField')) {
        self.$block.after($cloned);
      } else {
        $dataRows.append($cloned);
      }
      $cloned.find('input').first().focus();

      return false;
    },
    selectBlock: function (e) {
      var self = this;

      if (self.$block) {
        self.deselectBlock(e);
      }
      self.$block = $(e.currentTarget);
      self.openToolbox();
    },
    deselectBlock: function (e) {
      var self = this, $target;

      if (e) {
        $target = $(e.target);
        if ($target.parents().is('.selected')) {
          e.stopPropagation();
          return false;
        }
      }
      //self.focusTop = 0;
      self.focusId = null;

      if (self.$block) {
        self.$block.removeClass('selected');
        self.$block = null;
      }
      $('#containerToolbox').hide();
    },
    openToolbox: function () {
      var self = this, $toolbox, offset;
      if (self.$block === null) {
        return;
      }
      self.$block.addClass('selected');
      $toolbox = $('#containerToolbox');
      offset = self.$block.offset();
      offset.top += self.$block.outerHeight();
      offset.left += self.$block.outerWidth() - $toolbox.outerWidth();
      $toolbox.show().css({
        top: offset.top + 'px',
        left: offset.left + 'px'
      });
    },
    up: function () {
      var self = this;

      if (self.$block === null || self.$block.is(':first')) {
        return false;
      }
      self.$block.after(self.$block.prev());
      self.openToolbox();
    },
    down: function () {
      var self = this, $inputs;

      if (self.$block === null || self.$block.is(':last')) {
        return false;
      }
      self.$block.before(self.$block.next());
      $inputs = self.$block.find('input, select, textarea');
      $inputs.eq($inputs.length - 1).focus();

      $('#downField').focus();
      self.openToolbox();
    },
    remove: function (e) {
      var self = this;
      var $block = self.$block;

      if (!e.shiftKey && !confirm('データ（行）を削除しますか？')) {
        return;
      }

      $('#containerToolbox').hide();
      if ($block.next().length) {
        self.$block = $block.next();
        $block.remove();
        self.openToolbox();
      } else {
        self.deselectBlock(e);
        $block.remove();
      }
    },
    keyDown: function (e) {
      var self = this;

      switch (e.keyCode) {
        case 38: // up
          self.up();
          break;
        case 40: // down
          self.down();
          break;
      }
      return true;
    }
  };

})(jQuery, window);
