<?php
/**
 * public.zip 展開ツール
 * このスクリプトファイルをWebサーバーにアップロードすると、同じディレクトリ内のpublic.zipファイルを展開することができます。
 * 展開後はこのファイルとpublic.zipファイルは不要ですので、Webサーバーから削除してください。
 *
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

ini_set('memory_limit', '256M');
set_time_limit(300);

define('PUBLIC_ZIP', 'public.zip');

// 文字コード指定
ini_set('default_charset', 'UTF-8');
mb_internal_encoding('UTF-8');

// PHPバージョンの確認
if (version_compare(PHP_VERSION, '7.0.0') < 0) {
    exit_screen('アプリケーションエラー', 'このツールを実行するためにはPHP7.0.0以降が必要になります。');
}
// zip拡張がロードされていることを確認
if (!extension_loaded('zip')) {
    exit_screen('アプリケーションエラー', 'このツールを実行するためには zip 拡張が必要になります。');
}

// ファイルの存在確認
if (!file_exists(PUBLIC_ZIP)) {
    exit_screen('アプリケーションエラー', 'public.zip ファイルが見つかりません。');
}

// zipファイルから展開するパスのリスト
$extractPaths = array(
    'files/',
    'themes/',
    'system/',
    'index.php',
    'mecms_config.php',
    '.htaccess',
);
// 既存ファイル・ディレクトリ確認
foreach ($extractPaths as $path) {
    if (file_exists($path)) {
        exit_screen('アプリケーションエラー', '既存ファイル・ディレクトリが存在するため展開できません。パス：' . e($path));
    }
}
// 書き込みパーミッション確認
if (!is_writable('./')) {
    exit_screen('アプリケーションエラー', '現在のディレクトリに書き込み権限がないため展開できません。');
}

// 現在ディレクトリに展開する
$zip = new ZipArchive();
$res = $zip->open(PUBLIC_ZIP);
if ($res === TRUE) {
    $zip->extractTo('.');
    $zip->close();
} else {
    exit_screen('アプリケーションエラー', 'アーカイブファイルの展開に失敗しました。メッセージ：' . e($res));
}

// 成功画面を表示する
$html =  <<<EOD
<p>ファイルの展開に成功しました。</p>
<main><article>
<p>展開ツールと製品パッケージファイルを削除してください。</p>
<p>次のリンクをクリックしてインストール画面に移動してください。<br>
<a href="/admin/install">インストール</a></p>
</article></main>
EOD;

exit_screen('展開ツール', $html);

/**
 * HTMLエスケープ
 *
 * @param string $str
 * @return string
 */
function e($str)
{
    return htmlspecialchars($str, ENT_QUOTES, 'UTF-8');
}

/**
 * 画面をレンダリングしてスクリプトを終了する
 *
 * @param string $title
 * @param string $html
 * @return string
 */
function exit_screen(string $title, string $html)
{
    $escapedTitle = e($title);

    $content = <<<EOD
<!DOCTYPE html><html lang="ja"><head><meta charset="UTF-8"><title>{$escapedTitle}</title><meta name="viewport" content="width=device-width,initial-scale=1">
<style>body{margin:0;padding:15px}h1{margin:0 0 15px;font-size:24px;color:#F70}article{background-color:#eee;padding:10px 15px}article+article{margin:20px 0 0}h2{border-bottom:1px solid #ccc;margin:0 0 10px;font-size:18px;padding:0 0 5px}pre{background-color:#f7f7f7;padding:10px;margin:0 0 5px;overflow-x:scroll}</style>
</head><body>
<header><h1>{$escapedTitle}</h1></header>
{$html}
</body></html>
EOD;

    exit($content);
}
